module Data.Curve.Weierstrass.BLS12381T
  ( module Data.Curve.Weierstrass
  , Point(..)
  -- * BLS12381 curve
  , module Data.Curve.Weierstrass.BLS12381T
  ) where

import Protolude

import Data.Field.Galois
import GHC.Natural (Natural)

import Data.Curve.Weierstrass
import Data.Curve.Weierstrass.BLS12381 (BLS12381, Fq, Fr)

-------------------------------------------------------------------------------
-- Types
-------------------------------------------------------------------------------

-- | Field of points of BLS12381 curve over @Fq2@.
type Fq2 = Extension U Fq
data U
instance IrreducibleMonic U Fq where
  poly _ = [1, 0, 1]
  {-# INLINABLE poly #-}

-- BLS12381 curve is a Weierstrass curve.
instance Curve 'Weierstrass c BLS12381 Fq2 Fr => WCurve c BLS12381 Fq2 Fr where
  a_ = const _a
  {-# INLINABLE a_ #-}
  b_ = const _b
  {-# INLINABLE b_ #-}
  h_ = const _h
  {-# INLINABLE h_ #-}
  q_ = const _q
  {-# INLINABLE q_ #-}
  r_ = const _r
  {-# INLINABLE r_ #-}

-- | Affine BLS12381 curve point.
type PA = WAPoint BLS12381 Fq2 Fr

-- Affine BLS12381 curve is a Weierstrass affine curve.
instance WACurve BLS12381 Fq2 Fr where
  gA_ = gA
  {-# INLINABLE gA_ #-}

-- | Jacobian BLS12381 point.
type PJ = WJPoint BLS12381 Fq2 Fr

-- Jacobian BLS12381 curve is a Weierstrass Jacobian curve.
instance WJCurve BLS12381 Fq2 Fr where
  gJ_ = gJ
  {-# INLINABLE gJ_ #-}

-- | Projective BLS12381 point.
type PP = WPPoint BLS12381 Fq2 Fr

-- Projective BLS12381 curve is a Weierstrass projective curve.
instance WPCurve BLS12381 Fq2 Fr where
  gP_ = gP
  {-# INLINABLE gP_ #-}

-------------------------------------------------------------------------------
-- Parameters
-------------------------------------------------------------------------------

-- | Coefficient @A@ of BLS12381 curve.
_a :: Fq2
_a = toE' [
          ]
{-# INLINABLE _a #-}

-- | Coefficient @B@ of BLS12381 curve.
_b :: Fq2
_b = toE' [ 0x4
          , 0x4
          ]
{-# INLINABLE _b #-}

-- | Cofactor of BLS12381 curve.
_h :: Natural
_h = 0x5d543a95414e7f1091d50792876a202cd91de4547085abaa68a205b2e5a7ddfa628f1cb4d9e82ef21537e293a6691ae1616ec6e786f0c70cf1c38e31c7238e5
{-# INLINABLE _h #-}

-- | Characteristic of BLS12381 curve.
_q :: Natural
_q = 0x1a0111ea397fe69a4b1ba7b6434bacd764774b84f38512bf6730d2a0f6b0f6241eabfffeb153ffffb9feffffffffaaab
{-# INLINABLE _q #-}

-- | Order of BLS12381 curve.
_r :: Natural
_r = 0x73eda753299d7d483339d80809a1d80553bda402fffe5bfeffffffff00000001
{-# INLINABLE _r #-}

-- | Coordinate @X@ of BLS12381 curve.
_x :: Fq2
_x = toE' [ 0x24aa2b2f08f0a91260805272dc51051c6e47ad4fa403b02b4510b647ae3d1770bac0326a805bbefd48056c8c121bdb8
          , 0x13e02b6052719f607dacd3a088274f65596bd0d09920b61ab5da61bbdc7f5049334cf11213945d57e5ac7d055d042b7e
          ]
{-# INLINABLE _x #-}

-- | Coordinate @Y@ of BLS12381 curve.
_y :: Fq2
_y = toE' [ 0xce5d527727d6e118cc9cdc6da2e351aadfd9baa8cbdd3a76d429a695160d12c923ac9cc3baca289e193548608b82801
          , 0x606c4a02ea734cc32acd2b02bc28b99cb3e287e85a763af267492ab572e99ab3f370d275cec1da1aaa9075ff05f79be
          ]
{-# INLINABLE _y #-}

-- | Generator of affine BLS12381 curve.
gA :: PA
gA = A _x _y
{-# INLINABLE gA #-}

-- | Generator of Jacobian BLS12381 curve.
gJ :: PJ
gJ = J _x _y 1
{-# INLINABLE gJ #-}

-- | Generator of projective BLS12381 curve.
gP :: PP
gP = P _x _y 1
{-# INLINABLE gP #-}
